#!/sbin/sh
#
##############################################################
# File name       : installer.sh
#
# Description     : Setup installation, environmental variables and helper functions
#
# Build Date      : Friday March 15 11:36:43 IST 2019
#
# Updated on      : Sunday September 29 19:58:49 IST 2019
#
# GitHub          : TheHitMan7 <krtik.vrma@gmail.com>
#
# BiTGApps Author : TheHitMan @ xda-developers
#
# Copyright       : Copyright (C) 2019 TheHitMan7
#
# License         : SPDX-License-Identifier: GPL-3.0-or-later
#
# The BiTGApps scripts are free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version, w/BiTGApps installable zip exception.
#
# These scripts are distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
##############################################################

# Detect A/B partition layout https://source.android.com/devices/tech/ota/ab_updates
# and system-as-root https://source.android.com/devices/bootloader/system-as-root
#
# Set static partition layout. No matter which partition layout selected below
# "/system" is always mounted on top of "/system_root".
# Remount /system to /system_root if we have system-as-root
SYSTEM_MOUNT=/system_root
SYSTEM=$SYSTEM_MOUNT/system
SYSTEM_A_ONLY=/system
VENDOR=/vendor

# Set package defaults
TMP="/tmp"
TEMP="/cache"
LOGD="/cache/bitgapps"
LOG="/cache/bitgapps/installation.log"
ZIP_FILE="/tmp/zip"
UNZIP_DIR="/tmp/unzip"
TMP_ADDON="$UNZIP_DIR/tmp_addon"
TMP_SYS="$UNZIP_DIR/tmp_sys"
TMP_SYS_ROOT="$UNZIP_DIR/tmp_sys_root"
TMP_PRIV="$UNZIP_DIR/tmp_priv"
TMP_PRIV_ROOT="$UNZIP_DIR/tmp_priv_root"
TMP_LIB="$UNZIP_DIR/tmp_lib"
TMP_LIB64="$UNZIP_DIR/tmp_lib64"
TMP_FRAMEWORK="$UNZIP_DIR/tmp_framework"
TMP_CONFIG="$UNZIP_DIR/tmp_config"
TMP_DEFAULT_PERM="$UNZIP_DIR/tmp_default"
TMP_G_PERM="$UNZIP_DIR/tmp_perm"
TMP_G_PREF="$UNZIP_DIR/tmp_pref"
TMP_PERM_ROOT="$UNZIP_DIR/tmp_perm_root"
# Adapt installation instruction for A-only partition layout
if [ -f /system/init.rc ];
then
  SYSTEM_APP="$SYSTEM/product/app"
  SYSTEM_APP_ROOT="$SYSTEM/app"
  SYSTEM_PRIV_APP="$SYSTEM/product/priv-app"
  SYSTEM_PRIV_APP_ROOT="$SYSTEM/priv-app"
  SYSTEM_LIB="$SYSTEM/product/lib"
  SYSTEM_LIB64="$SYSTEM/product/lib64"
  SYSTEM_ADDOND="$SYSTEM/addon.d"
  SYSTEM_FRAMEWORK="$SYSTEM/framework"
  SYSTEM_ETC_CONFIG="$SYSTEM/product/etc/sysconfig"
  SYSTEM_ETC_DEFAULT="$SYSTEM/product/etc/default-permissions"
  SYSTEM_ETC_PERM="$SYSTEM/product/etc/permissions"
  SYSTEM_ETC_PREF="$SYSTEM/product/etc/preferred-apps"
  SYSTEM_ETC_PERM_ROOT="$SYSTEM/etc/permissions"
else
  SYSTEM_APP="$SYSTEM_A_ONLY/product/app"
  SYSTEM_APP_ROOT="$SYSTEM_A_ONLY/app"
  SYSTEM_PRIV_APP="$SYSTEM_A_ONLY/product/priv-app"
  SYSTEM_PRIV_APP_ROOT="$SYSTEM_A_ONLY/priv-app"
  SYSTEM_LIB="$SYSTEM_A_ONLY/product/lib"
  SYSTEM_LIB64="$SYSTEM_A_ONLY/product/lib64"
  SYSTEM_ADDOND="$SYSTEM_A_ONLY/addon.d"
  SYSTEM_FRAMEWORK="$SYSTEM_A_ONLY/framework"
  SYSTEM_ETC_CONFIG="$SYSTEM_A_ONLY/product/etc/sysconfig"
  SYSTEM_ETC_DEFAULT="$SYSTEM_A_ONLY/product/etc/default-permissions"
  SYSTEM_ETC_PERM="$SYSTEM_A_ONLY/product/etc/permissions"
  SYSTEM_ETC_PREF="$SYSTEM_A_ONLY/product/etc/preferred-apps"
  SYSTEM_ETC_PERM_ROOT="$SYSTEM_A_ONLY/etc/permissions"
fi
EXTRA_LOG="/cache/bitgapps/extra.log"
SQLITE_LOG="/cache/bitgapps/sqlite.log"
SQLITE_TOOL="/tmp/sqlite3"

if [ -d "$LOGD" ]; then
   echo "- Log directory found in :" $TEMP
else
   echo "- Log directory not found in :" $TEMP
   echo "- Creating in :" $TEMP
   mkdir /cache/bitgapps
   chmod 1755 /cache/bitgapps
   echo "- Done"
fi

# Creating installation components
service_manager() {
 echo "-----------------------------------" >> $LOG
 echo " --- BiTGApps Installation Log --- " >> $LOG
 echo "             Start at              " >> $LOG
 echo "        $( date +"%m-%d-%Y %H:%M:%S" )" >> $LOG
 echo "-----------------------------------" >> $LOG
 echo " " >> $LOG
 echo "-----------------------------------" >> $LOG
 if [ -d "$LOGD" ]; then
    echo "- Log directory found in :" $TEMP >> $LOG
 else
    echo "- Log directory not found in" $TEMP >> $LOG
 fi
 echo "-----------------------------------" >> $LOG
 if [ -d "$UNZIP_DIR" ]; then
    echo "- Unzip directory found in :" $TMP >> $LOG
    echo "- Creating components in :" $TMP >> $LOG
    mkdir $UNZIP_DIR/tmp_addon
    mkdir $UNZIP_DIR/tmp_sys
    mkdir $UNZIP_DIR/tmp_sys_root
    mkdir $UNZIP_DIR/tmp_priv
    mkdir $UNZIP_DIR/tmp_priv_root
    mkdir $UNZIP_DIR/tmp_lib
    mkdir $UNZIP_DIR/tmp_lib64
    mkdir $UNZIP_DIR/tmp_framework
    mkdir $UNZIP_DIR/tmp_config
    mkdir $UNZIP_DIR/tmp_default
    mkdir $UNZIP_DIR/tmp_perm
    mkdir $UNZIP_DIR/tmp_pref
    mkdir $UNZIP_DIR/tmp_perm_root
    chmod 0755 $UNZIP_DIR
    chmod 0755 $UNZIP_DIR/tmp_addon
    chmod 0755 $UNZIP_DIR/tmp_sys
    chmod 0755 $UNZIP_DIR/tmp_sys_root
    chmod 0755 $UNZIP_DIR/tmp_priv
    chmod 0755 $UNZIP_DIR/tmp_priv_root
    chmod 0755 $UNZIP_DIR/tmp_lib
    chmod 0755 $UNZIP_DIR/tmp_lib64
    chmod 0755 $UNZIP_DIR/tmp_framework
    chmod 0755 $UNZIP_DIR/tmp_config
    chmod 0755 $UNZIP_DIR/tmp_default
    chmod 0755 $UNZIP_DIR/tmp_perm
    chmod 0755 $UNZIP_DIR/tmp_pref
    chmod 0755 $UNZIP_DIR/tmp_perm_root
 else
   echo "- Unzip directory not found in :" $TMP >> $LOG
 fi
}

# Removing pre-installed system files
pre_installed() {
 rm -rf $SYSTEM_APP_ROOT/ExtShared
 rm -rf $SYSTEM_APP_ROOT/GoogleExtShared
 rm -rf $SYSTEM_APP/GoogleContactsSyncAdapter
 rm -rf $SYSTEM_APP/LocationHistoryPrebuilt
 rm -rf $SYSTEM_APP/MarkupGoogle
 rm -rf $SYSTEM_PRIV_APP/AndroidPlatformServices
 rm -rf $SYSTEM_PRIV_APP/ConfigUpdater
 rm -rf $SYSTEM_PRIV_APP_ROOT/ExtServices
 rm -rf $SYSTEM_PRIV_APP_ROOT/GoogleExtServicesPrebuilt
 rm -rf $SYSTEM_PRIV_APP/GoogleServicesFramework
 rm -rf $SYSTEM_PRIV_APP/Phonesky
 rm -rf $SYSTEM_PRIV_APP/PrebuiltGmsCoreQt
 rm -rf $SYSTEM_FRAMEWORK/com.google.android.dialer.support.jar
 rm -rf $SYSTEM_FRAMEWORK/com.google.android.maps.jar
 rm -rf $SYSTEM_FRAMEWORK/com.google.android.media.effects.jar
 rm -rf $SYSTEM_LIB/libsketchology_native.so
 rm -rf $SYSTEM_LIB64/libsketchology_native.so
 rm -rf $SYSTEM_ETC_CONFIG/google.xml
 rm -rf $SYSTEM_ETC_CONFIG/google_build.xml
 rm -rf $SYSTEM_ETC_CONFIG/google-hiddenapi-package-whitelist.xml
 rm -rf $SYSTEM_ETC_CONFIG/google_vr_build.xml
 rm -rf $SYSTEM_ETC_CONFIG/turbo.xml
 rm -rf $SYSTEM_ETC_CONFIG/wellbeing.xml
 rm -rf $SYSTEM_ETC_CONFIG/whitelist_com.android.omadm.service.xml
 rm -rf $SYSTEM_ETC_DEFAULT
 rm -rf $SYSTEM_ETC_PERM/com.google.android.dialer.support.xml
 rm -rf $SYSTEM_ETC_PERM/com.google.android.maps.xml
 rm -rf $SYSTEM_ETC_PERM/com.google.android.media.effects.xml
 rm -rf $SYSTEM_ETC_PERM/GoogleExtServices_permissions.xml
 rm -rf $SYSTEM_ETC_PERM/privapp-permissions-google.xml
 rm -rf $SYSTEM_ETC_PERM/privapp-permissions-google-p.xml
 rm -rf $SYSTEM_ETC_PERM/privapp-permissions-google-ps.xml
 rm -rf $SYSTEM_ETC_PERM/privapp-permissions-google-system.xml
 rm -rf $SYSTEM_ETC_PERM/split-permissions-google.xml
 rm -rf $SYSTEM_ETC_PREF
 rm -rf $SYSTEM_ETC_PERM_ROOT/privapp-permissions-google.xml
}

# Unpack system files
extract_app() {
 echo "-----------------------------------" >> $LOG
 echo "- Unpack SYS-APP Files" >> $LOG
 unzip -o $ZIP_FILE/sys/sys_app_GoogleContactsSyncAdapter.zip -d $TMP_SYS >> $LOG
 unzip -o $ZIP_FILE/sys/sys_app_GoogleExtShared.zip -d $TMP_SYS_ROOT >> $LOG
 unzip -o $ZIP_FILE/sys/sys_app_LocationHistoryPrebuilt.zip -d $TMP_SYS >> $LOG
 unzip -o $ZIP_FILE/sys/sys_app_MarkupGoogle.zip -d $TMP_SYS >> $LOG
 echo "- Done" >> $LOG
 echo "-----------------------------------" >> $LOG
 echo "- Unpack PRIV-APP Files" >> $LOG
 unzip -o $ZIP_FILE/core/priv_app_AndroidPlatformServices.zip -d $TMP_PRIV >> $LOG
 unzip -o $ZIP_FILE/core/priv_app_ConfigUpdater.zip -d $TMP_PRIV >> $LOG
 unzip -o $ZIP_FILE/core/priv_app_GoogleExtServicesPrebuilt.zip -d $TMP_PRIV_ROOT >> $LOG
 unzip -o $ZIP_FILE/core/priv_app_GoogleServicesFramework.zip -d $TMP_PRIV >> $LOG
 unzip -o $ZIP_FILE/core/priv_app_Phonesky.zip -d $TMP_PRIV >> $LOG
 unzip -o $ZIP_FILE/core/priv_app_PrebuiltGmsCoreQt.zip -d $TMP_PRIV >> $LOG
 echo "- Done" >> $LOG
 echo "-----------------------------------" >> $LOG
 echo "- Unpack Framework Files" >> $LOG
 unzip -o $ZIP_FILE/sys_Framework.zip -d $TMP_FRAMEWORK >> $LOG
 echo "- Done" >> $LOG
 echo "-----------------------------------" >> $LOG
 echo "- Unpack System Lib" >> $LOG
 unzip -o $ZIP_FILE/sys_Lib.zip -d $TMP_LIB >> $LOG
 echo "- Done" >> $LOG
 echo "-----------------------------------" >> $LOG
 echo "- Unpack System Lib64" >> $LOG
 unzip -o $ZIP_FILE/sys_Lib64.zip -d $TMP_LIB64 >> $LOG
 echo "- Done" >> $LOG
 echo "-----------------------------------" >> $LOG
 echo "- Unpack System Files" >> $LOG
 unzip -o $ZIP_FILE/sys_Config_Permission.zip -d $TMP_CONFIG >> $LOG
 unzip -o $ZIP_FILE/sys_Default_Permission.zip -d $TMP_DEFAULT_PERM >> $LOG
 unzip -o $ZIP_FILE/sys_Permissions.zip -d $TMP_G_PERM >> $LOG
 unzip -o $ZIP_FILE/sys_Pref_Permission.zip -d $TMP_G_PREF >> $LOG
 unzip -o $ZIP_FILE/sys_root_Permissions.zip -d $TMP_PERM_ROOT >> $LOG
 echo "- Done" >> $LOG
 echo "-----------------------------------" >> $LOG
 echo "- Installation Complete" >> $LOG
 echo "-----------------------------------" >> $LOG
 echo "Finish at $( date +"%m-%d-%Y %H:%M:%S" )" >> $LOG
 echo "-----------------------------------" >> $LOG
}

# Install required configs
config_install() {
 echo "-----------------------------------" >> $EXTRA_LOG;
 if [ -d "$SYSTEM_ETC_PREF" ]; then
    echo "- Build dependency found in : /system" >> $EXTRA_LOG;
 else
    echo "- Build dependency not found in : /system" >> $EXTRA_LOG;
    echo "- Creating in : /system" >> $EXTRA_LOG;
	  if [ -f /system/init.rc ];
	  then
      mkdir $SYSTEM/product/etc/preferred-apps
      chmod 0755 $SYSTEM/product/etc/preferred-apps
	  else
	    mkdir $SYSTEM_A_ONLY/product/etc/preferred-apps
      chmod 0755 $SYSTEM_A_ONLY/product/etc/preferred-apps
	  fi
    echo "- Done" >> $EXTRA_LOG;
 fi
 echo "-----------------------------------" >> $EXTRA_LOG;
 if [ -d "$SYSTEM_ETC_DEFAULT" ]; then
    echo "- Build dependency found in : /system" >> $EXTRA_LOG;
 else
    echo "- Build dependency not found in : /system" >> $EXTRA_LOG;
    echo "- Creating in : /system" >> $EXTRA_LOG;
	  if [ -f /system/init.rc ];
	  then
      mkdir $SYSTEM/product/etc/default-permissions
      chmod 0755 $SYSTEM/product/etc/default-permissions
	  else
	    mkdir $SYSTEM_A_ONLY/product/etc/default-permissions
      chmod 0755 $SYSTEM_A_ONLY/product/etc/default-permissions
	  fi
    echo "- Done" >> $EXTRA_LOG;
 fi
 echo "-----------------------------------" >> $EXTRA_LOG;
 if [ -d "$SYSTEM_LIB" ]; then
    echo "- Build dependency found : /system" >> $EXTRA_LOG;
 else
    echo "- Build dependency not found in : /system" >> $EXTRA_LOG;
    echo "- Creating in : /system" >> $EXTRA_LOG;
	  if [ -f /system/init.rc ];
	  then
      mkdir $SYSTEM/product/lib
      chmod 0755 $SYSTEM/product/lib
	  else
	    mkdir $SYSTEM_A_ONLY/product/lib
	    chmod 0755 $SYSTEM_A_ONLY/product/lib
	  fi
    echo "- Done" >> $EXTRA_LOG;
 fi
 echo "-----------------------------------" >> $EXTRA_LOG;
}

selinux_context_s1() {
 chcon -h u:object_r:system_file:s0 "$SYSTEM_APP/GoogleContactsSyncAdapter";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_APP_ROOT/GoogleExtShared";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_APP/LocationHistoryPrebuilt";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_APP/MarkupGoogle";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_APP/GoogleContactsSyncAdapter/GoogleContactsSyncAdapter.apk";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_APP_ROOT/GoogleExtShared/GoogleExtShared.apk";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_APP/LocationHistoryPrebuilt/LocationHistoryPrebuilt.apk";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_APP/MarkupGoogle/MarkupGoogle.apk";
}

selinux_context_sp2() {
 chcon -h u:object_r:system_file:s0 "$SYSTEM_PRIV_APP/AndroidPlatformServices";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_PRIV_APP/ConfigUpdater";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_PRIV_APP_ROOT/GoogleExtServicesPrebuilt";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_PRIV_APP/GoogleServicesFramework";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_PRIV_APP/Phonesky";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_PRIV_APP/PrebuiltGmsCoreQt";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_PRIV_APP/AndroidPlatformServices/AndroidPlatformServices.apk";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_PRIV_APP/ConfigUpdater/ConfigUpdater.apk";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_PRIV_APP_ROOT/GoogleExtServicesPrebuilt/GoogleExtServicesPrebuilt.apk";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_PRIV_APP/GoogleServicesFramework/GoogleServicesFramework.apk";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_PRIV_APP/Phonesky/Phonesky.apk";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_PRIV_APP/PrebuiltGmsCoreQt/PrebuiltGmsCoreQt.apk";
}

selinux_context_sf3() {
 chcon -h u:object_r:system_file:s0 "$SYSTEM_FRAMEWORK/com.google.android.dialer.support.jar";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_FRAMEWORK/com.google.android.maps.jar";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_FRAMEWORK/com.google.android.media.effects.jar";
}

selinux_context_sl4() {
 chcon -h u:object_r:system_file:s0 "$SYSTEM_LIB/libsketchology_native.so";
}

selinux_context_sl5() {
 chcon -h u:object_r:system_file:s0 "$SYSTEM_LIB64/libsketchology_native.so";
}

selinux_context_se6() {
 if [ -f /system/init.rc ];
 then
   chcon -h u:object_r:system_file:s0 "$SYSTEM/product/etc/default-permissions";
 else
   chcon -h u:object_r:system_file:s0 "$SYSTEM_A_ONLY/product/etc/default-permissions";
 fi
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_DEFAULT/default-permissions-google.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_DEFAULT/opengapps-permissions.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_PERM/com.google.android.dialer.support.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_PERM/com.google.android.maps.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_PERM/com.google.android.media.effects.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_PERM/GoogleExtServices_permissions.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_PERM/privapp-permissions-google.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_PERM/privapp-permissions-google-p.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_PERM/privapp-permissions-google-ps.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_PERM/privapp-permissions-google-system.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_PERM/split-permissions-google.xml";
 if [ -f /system/init.rc ];
 then
   chcon -h u:object_r:system_file:s0 "$SYSTEM/product/etc/preferred-apps";
 else
   chcon -h u:object_r:system_file:s0 "$SYSTEM_A_ONLY/product/etc/preferred-apps";
 fi
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_PREF/google.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_CONFIG/google.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_CONFIG/google_build.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_CONFIG/google-hiddenapi-package-whitelist.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_CONFIG/google_vr_build.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_CONFIG/turbo.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_CONFIG/wellbeing.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_CONFIG/whitelist_com.android.omadm.service.xml";
 chcon -h u:object_r:system_file:s0 "$SYSTEM_ETC_PERM_ROOT/privapp-permissions-google.xml";
 if [ -f /system/init.rc ];
 then
   chcon -h u:object_r:system_file:s0 "$SYSTEM/etc/g.prop";
 else
   chcon -h u:object_r:system_file:s0 "$SYSTEM_A_ONLY/etc/g.prop";
 fi
}

## End selinux method

# Complete the installation in sparse package format
send_sparse_1() {
 file_list="$(find "$TMP_SYS/" -mindepth 1 -type f | cut -d/ -f5-)"
 dir_list="$(find "$TMP_SYS/" -mindepth 1 -type d | cut -d/ -f5-)"
 for file in $file_list; do
     install -D "$TMP_SYS/${file}" "$SYSTEM_APP/${file}"
     chmod 0644 "$SYSTEM_APP/${file}";
 done
 for dir in $dir_list; do
     chmod 0755 "$SYSTEM_APP/${dir}";
 done
}

send_sparse_2() {
 file_list="$(find "$TMP_SYS_ROOT/" -mindepth 1 -type f | cut -d/ -f5-)"
 dir_list="$(find "$TMP_SYS_ROOT/" -mindepth 1 -type d | cut -d/ -f5-)"
 for file in $file_list; do
     install -D "$TMP_SYS_ROOT/${file}" "$SYSTEM_APP_ROOT/${file}"
     chmod 0644 "$SYSTEM_APP_ROOT/${file}";
 done
 for dir in $dir_list; do
     chmod 0755 "$SYSTEM_APP_ROOT/${dir}";
 done
}

send_sparse_3() {
 file_list="$(find "$TMP_PRIV/" -mindepth 1 -type f | cut -d/ -f5-)"
 dir_list="$(find "$TMP_PRIV/" -mindepth 1 -type d | cut -d/ -f5-)"
 for file in $file_list; do
     install -D "$TMP_PRIV/${file}" "$SYSTEM_PRIV_APP/${file}"
     chmod 0644 "$SYSTEM_PRIV_APP/${file}";
 done
 for dir in $dir_list; do
     chmod 0755 "$SYSTEM_PRIV_APP/${dir}";
 done
}

send_sparse_4() {
 file_list="$(find "$TMP_PRIV_ROOT/" -mindepth 1 -type f | cut -d/ -f5-)"
 dir_list="$(find "$TMP_PRIV_ROOT/" -mindepth 1 -type d | cut -d/ -f5-)"
 for file in $file_list; do
     install -D "$TMP_PRIV_ROOT/${file}" "$SYSTEM_PRIV_APP_ROOT/${file}"
     chmod 0644 "$SYSTEM_PRIV_APP_ROOT/${file}";
 done
 for dir in $dir_list; do
     chmod 0755 "$SYSTEM_PRIV_APP_ROOT/${dir}";
 done
}

send_sparse_5() {
 file_list="$(find "$TMP_FRAMEWORK/" -mindepth 1 -type f | cut -d/ -f5-)"
 dir_list="$(find "$TMP_FRAMEWORK/" -mindepth 1 -type d | cut -d/ -f5-)"
 for file in $file_list; do
     install -D "$TMP_FRAMEWORK/${file}" "$SYSTEM_FRAMEWORK/${file}"
     chmod 0644 "$SYSTEM_FRAMEWORK/${file}";
 done
 for dir in $dir_list; do
   chmod 0755 "$SYSTEM_FRAMEWORK/${dir}";
 done
}

send_sparse_6() {
 file_list="$(find "$TMP_LIB/" -mindepth 1 -type f | cut -d/ -f5-)"
 dir_list="$(find "$TMP_LIB/" -mindepth 1 -type d | cut -d/ -f5-)"
 for file in $file_list; do
     install -D "$TMP_LIB/${file}" "$SYSTEM_LIB/${file}"
     chmod 0644 "$SYSTEM_LIB/${file}";
 done
 for dir in $dir_list; do
   chmod 0755 "$SYSTEM_LIB/${dir}";
 done
}

send_sparse_7() {
 file_list="$(find "$TMP_LIB64/" -mindepth 1 -type f | cut -d/ -f5-)"
 dir_list="$(find "$TMP_LIB64/" -mindepth 1 -type d | cut -d/ -f5-)"
 for file in $file_list; do
     install -D "$TMP_LIB64/${file}" "$SYSTEM_LIB64/${file}"
     chmod 0644 "$SYSTEM_LIB64/${file}";
 done
 for dir in $dir_list; do
   chmod 0755 "$SYSTEM_LIB64/${dir}";
 done
}

send_sparse_8() {
 file_list="$(find "$TMP_CONFIG/" -mindepth 1 -type f | cut -d/ -f5-)"
 dir_list="$(find "$TMP_CONFIG/" -mindepth 1 -type d | cut -d/ -f5-)"
 for file in $file_list; do
     install -D "$TMP_CONFIG/${file}" "$SYSTEM_ETC_CONFIG/${file}"
     chmod 0644 "$SYSTEM_ETC_CONFIG/${file}";
 done
 for dir in $dir_list; do
   chmod 0755 "$SYSTEM_ETC_CONFIG/${dir}";
 done
}

send_sparse_9() {
 file_list="$(find "$TMP_DEFAULT_PERM/" -mindepth 1 -type f | cut -d/ -f5-)"
 dir_list="$(find "$TMP_DEFAULT_PERM/" -mindepth 1 -type d | cut -d/ -f5-)"
 for file in $file_list; do
     install -D "$TMP_DEFAULT_PERM/${file}" "$SYSTEM_ETC_DEFAULT/${file}"
     chmod 0644 "$SYSTEM_ETC_DEFAULT/${file}";
 done
 for dir in $dir_list; do
   chmod 0755 "$SYSTEM_ETC_DEFAULT/${dir}";
 done
}

send_sparse_10() {
 file_list="$(find "$TMP_G_PREF/" -mindepth 1 -type f | cut -d/ -f5-)"
 dir_list="$(find "$TMP_G_PREF/" -mindepth 1 -type d | cut -d/ -f5-)"
 for file in $file_list; do
     install -D "$TMP_G_PREF/${file}" "$SYSTEM_ETC_PREF/${file}"
     chmod 0644 "$SYSTEM_ETC_PREF/${file}";
 done
 for dir in $dir_list; do
   chmod 0755 "$SYSTEM_ETC_PREF/${dir}";
 done
}

send_sparse_11() {
 file_list="$(find "$TMP_G_PERM/" -mindepth 1 -type f | cut -d/ -f5-)"
 dir_list="$(find "$TMP_G_PERM/" -mindepth 1 -type d | cut -d/ -f5-)"
 for file in $file_list; do
     install -D "$TMP_G_PERM/${file}" "$SYSTEM_ETC_PERM/${file}"
     chmod 0644 "$SYSTEM_ETC_PERM/${file}";
 done
 for dir in $dir_list; do
   chmod 0755 "$SYSTEM_ETC_PERM/${dir}";
 done
}

send_sparse_12() {
 file_list="$(find "$TMP_PERM_ROOT/" -mindepth 1 -type f | cut -d/ -f5-)"
 dir_list="$(find "$TMP_PERM_ROOT/" -mindepth 1 -type d | cut -d/ -f5-)"
 for file in $file_list; do
     install -D "$TMP_PERM_ROOT/${file}" "$SYSTEM_ETC_PERM_ROOT/${file}"
     chmod 0644 "$SYSTEM_ETC_PERM_ROOT/${file}";
 done
 for dir in $dir_list; do
   chmod 0755 "$SYSTEM_ETC_PERM_ROOT/${dir}";
 done
}

send_sparse_13() {
 if [ -f /system/init.rc ];
 then
   cp -f $TMP/g.prop $SYSTEM/etc/g.prop
   chmod 0644 $SYSTEM/etc/g.prop
 else
   cp -f $TMP/g.prop $SYSTEM_A_ONLY/etc/g.prop
   chmod 0644 $SYSTEM_A_ONLY/etc/g.prop
 fi
}

# execute installation functions
service_manager;
pre_installed;
config_install;
extract_app;
# sending packages in sparse mode
send_sparse_1;
send_sparse_2;
send_sparse_3;
send_sparse_4;
send_sparse_5;
send_sparse_6;
send_sparse_7;
send_sparse_8;
send_sparse_9;
send_sparse_10;
send_sparse_11;
send_sparse_12;
send_sparse_13;
# execute selinux install functions
selinux_context_s1;
selinux_context_sp2;
selinux_context_sf3;
selinux_context_sl4;
selinux_context_sl5;
selinux_context_se6;

## End method
